@@ -1,4 +1,4 @@
 #define MFBIMAGEGLYPHBLT xf1bppImageGlyphBltWhite
 #define OPEQ |=
 #include "mfbmap.h"
-#include "../../../../mfb/mfbimggblt.c"
+#include "../../../mfb/mfbimggblt.c"
