@@ -2,4 +2,4 @@
 #define OPEQ ^=
 #define EQWHOLEWORD ^=~0
 #include "mfbmap.h"
-#include "../../../../mfb/mfbply1rct.c"
+#include "../../../mfb/mfbply1rct.c"
