@@ -1,3 +1,3 @@
 #define POLYSEGMENT
 #include "mfbmap.h"
-#include "../../../../mfb/mfbline.c"
+#include "../../../mfb/mfbline.c"
