@@ -222,7 +222,8 @@ PixmapPtr
 xglCreatePixmap (ScreenPtr  pScreen,
 		 int	    width,
 		 int	    height,
-		 int	    depth)
+		 int	    depth,
+		 unsigned   usage_hint)
 {
     xglPixmapPtr pPixmapPriv;
     PixmapPtr	 pPixmap;
@@ -253,6 +254,7 @@ xglCreatePixmap (ScreenPtr  pScreen,
     pPixmap->devKind = 0;
     pPixmap->refcnt = 1;
     pPixmap->devPrivate.ptr = 0;
+    pPixmap->usage_hint = usage_hint;
 
     pPixmapPriv = XGL_GET_PIXMAP_PRIV (pPixmap);
 
@@ -310,6 +312,7 @@ xglDestroyPixmap (PixmapPtr pPixmap)
 
     xglFiniPixmap (pPixmap);
 
+    dixFreePrivates(pPixmap->devPrivates);
     xfree (pPixmap);
 
     return TRUE;
